
var websocket;
var interval;
var onFileUpload = false;
var windowFocus = true;
var typingTimeOut;

var chatroom = {
	opt : {
		title : document.title,
		typing : false
	},
	selectors : {
		$idIsComment : $("#is-comment"),
		$idInsertComment : $('#insert-comment'),
		$idRoomWhoType : $("#room-who-type"),
		$chatroomItemList : $(".chatroom-item-list"),
		$idCommentList : $("#comment-list"),
		$chatroomMagListScroll : $(".chatroom-msg-list-scroll"),
		$inputPoint : $("input[name='point']"),
		$idCommentBox : $('#is-comment-box'),
		$msgHeader : $(".msg-header")
	},
	init : function( callback ){
		/** 	初始化	 */
		toggleInputArea(true);
		$(".msg-header-other-name").html( chatroom.opt.otherName );
		chatroom.initWebsocket(chatroom.opt.wsUrl , callback);
	},
	initWebsocket : function(wsurl, callback){
		/** 	websocket 初始化	 */
		websocket = new ReconnectingWebSocket(wsurl);
		websocket.onopen = function(evt){
			chatroom.onopen();
			callback();
		}
		
		websocket.onmessage = function(evt){
			chatroom.opt.alive = true;
			
			if("pong" == evt.data){
				return;
			}

			//console.log( evt.data );
			var obj = JSON.parse(evt.data);

			if ( "readed" == obj.action ) {
				chatroom.countUnRead();
				return;
			}
			
			if ( obj.action == 'countUnRead' ) {
				//	最新未讀數量
				chatroom.refreshTitle( obj );
				
			} else if ( obj.roomId != chatroom.opt.roomId ) {
				//	不同聊天室
				chatroom.onotherroom(obj);
				
			} else {
				//	同聊天室
				
				if ( ( "insert" == obj.action && obj.senderId == chatroom.opt.currentUserId ) ) {
					resetInsert();
				}
				
				if( obj.action == 'typing' ){
					//	輸入中
					chatroom.ontyping( obj.senderName );
					
				} else if ( obj.action == 'stopTyping' ){
					//	停止輸入
					chatroom.onstoptyping( obj.senderName );
					
				}else if( obj.action == 'ACK' ){
					//	訊息接收成功
					chatroom.onack(obj);
					
				}else{
					//	其他，新增，修改，刪除，申訴，訊息清單，錯誤訊息
					chatroom.onmessage(obj);
				}
			}

		}
		
		websocket.onclose = function(evt) {
			toggleInputArea(true);
		}
		
		websocket.onerror = chatroom.disconnected;
		
		window.onbeforeunload = function(){
			websocket.onclose = function(){};
			websocket.close();
		}
	},
	onopen : function(){
		/**		websocket 建立時	*/
		toggleInputArea(false);
		chatroom.testAlive();
		
		//	最新未讀數量
		chatroom.countUnRead();
		
		chatroom.selectors.$idIsComment.keyup(function(){
			colseEdit();
			var val = $(this).val();
			if ( val.length == 0 ) {
				//	沒有內容
				if ( chatroom.opt.typing ) {
					//	已經是輸入中，進入 stopTyping
					var obj = chatroom.createMessagePack('stopTyping');
					chatroom.send(obj);
					chatroom.opt.typing = false;
					if ( typingTimeOut ) {
						clearTimeout( typingTimeOut );
					}
				}
				
			}else{
				//	有內容
				if ( !chatroom.opt.typing ) {
					//	不是輸入中，進入 typing
					var obj = chatroom.createMessagePack('typing');
					chatroom.send(obj);
					chatroom.opt.typing = true;
				} 
				
				if ( typingTimeOut ) {
					clearTimeout( typingTimeOut );
				}
				
				//	5秒後，進入stopTyping
				typingTimeOut = setTimeout( function(){
					
					if ( chatroom.opt.typing ) {
						var obj = chatroom.createMessagePack('stopTyping');
						chatroom.send(obj);
						chatroom.opt.typing = false;
					}
				} , 5000 );
			}
		});
		
	},
	testAlive : function(){
		/**		保持 websocket 是否通暢	*/
		setInterval( function(){ websocket.send("ping"); } , 2500);
	},
	createMessagePack : function(action){
		/**		建立傳送資料的物件	*/
		var obj = {
			action : action,
			senderId : chatroom.opt.currentUserId,
			senderName : chatroom.opt.currentUserName,
			roomId : chatroom.opt.roomId,
			tab : chatroom.opt.tab
		};
		
		if( action == 'sendMessage' || action == 'sendFileMessage' ){
			obj.messageId = chatroom.createMsgId();
		}
	
		return obj;
	},
	createMsgId : function(){
		/**		產生新訊息的編號	*/
		var midPlain = chatroom.opt.roomId + "" + chatroom.opt.currentUserId + "" + Date.now() + "" + Math.random();
		var shaObj = new jsSHA(midPlain, "TEXT");
		return shaObj.getHash("SHA-256", "HEX");
	},
	send : function(obj){
		/**		送出資料	*/
		websocket.send(JSON.stringify(obj));
		chatroom.opt.alive = true;
	},
	onack : function(obj){},
	ontyping : function( who ){
		/**		輸入中	*/
		chatroom.selectors.$idRoomWhoType.find("span").text(who + i18n.view.isTyping ).parent().show();
	},
	onstoptyping : function( who ){
		/**		停止輸入	*/
		chatroom.selectors.$idRoomWhoType.find("span").text("").parent().hide();
	},
	onmessage : function( obj ){
		/**		資料處理	*/
		if ( "fail" == obj.action ) {
			toastr.clear();
			toastr.warning( obj.respones );
		} else {
			
			if( obj.messageId ){
				
				var $list = chatroom.selectors.$idCommentList;
				
				if ( "insert" == obj.action ) {
					//	新增訊息
					var $box = genCommentHtml( obj );
					var $last = $( ".same-poster:last" , $list );
					
					var preTZ = $(" .comment:last" , $list ).attr("data-timestamp");
					var nowTZ = $(" .comment" , $box ).attr("data-timestamp");
					
					var preDate = new Date( parseInt( preTZ ) ).toLocaleDateString(),
						today = new Date().toLocaleDateString();

					var onCross = false;
					if ( preDate != today ) {
						//		上一個訊息不是今天的，表示跨日，顯示今天的時間線
						var $dateLine = $("#date-line").clone();
						$dateLine.removeClass("hidden")
							.addClass( $dateLine.attr("id") )
							.removeAttr("id")
							.find("div").html( i18n.view.dateLineToday );
						$list.append( $dateLine );
						onCross = true;
					}
					
					if ( obj.point > 0 || $box.attr("data-sender") != $last.attr("data-sender") ||
							$last.hasClass("need-point") ) {
							//	需要付費 或 換人
							$list.append( $box );
							
					} else {

						if ( nowTZ - preTZ > 180000 || onCross ) {
							//		與上個訊息相差超過3分鐘
							$list.append( $box );
						} else {
							$(" .commont-grop" , $last ).append( $("[data-id]" , $box ) );
							$(" .newest-comment-date" , $last ).html( $(" .time-ago-text" , $box ) );
						}
					}
					
					setScrollHeight( function(){
						chatroom.selectors.$chatroomMagListScroll.scrollTo( 9999999999 );
					} );
					
					
					if ( !windowFocus ) {
						if ( interval ) {
							clearInterval( interval );
						}
						var str = obj.senderName+" "+i18n.view.sendNewMsg;
						interval = setInterval( "flashTitle('"+document.title+"','"+str+"')" , 1000 );
						
					} else {
						var obj = chatroom.createMessagePack("readed");
						chatroom.send(obj);
					}
					
				} else if ( "update" == obj.action ) {
					//	更新訊息
					var $box = genCommentHtml( obj );
					var $comment = $list.find("[data-id="+obj.messageId+"]");
					
					if ( obj.point > 0 ) {
						$comment = $comment.parents(".same-poster");
						$comment.html( $( " > " , $box ) );
					} else {
						$comment.after( $("[data-id]" , $box ) );
						$comment.remove();
					}
					
				} else if ( "delete" == obj.action ) {
					//	刪除檔案
					var $comment = $list.find("[data-id="+obj.messageId+"]");
					var $poster = $comment.parents(".same-poster");
					var count = $poster.find("[data-id]").length;
					
					if ( count > 1 ) {
						$comment.remove();
					} else {
						$poster.remove();
					}
				}
				
			} else {
				
				if ( "initMessage" == obj.action ) {

					if ( obj.senderId != chatroom.opt.currentUserId ) {
						//	不是自己的頁面資料，不處理
						return;
					}
					
					//	聊天室初始資料倒入
					if ( obj.list != null ) {
						//	有資料
						chatroom.selectors.$chatroomItemList
							.find("[data-id="+ obj.roomId +"]")
							.find(".chatroom-more")
							.attr("data-count" , 0 ).text("");
						
						resetTitleWithNewMsgCount();
						
						chatroom.reload( obj.list , function ( $clone ) {
							chatroom.selectors.$idCommentList.html( $clone.find(" > .same-poster ") );
							$('.time-ago-text').timeago();
							
							setScrollHeight( function(){
								chatroom.selectors.$chatroomMagListScroll.scrollTo( 999999999999 );
							} );
							
						} );
						
					} else {
						chatroom.selectors.$idCommentList.html( "" );
					}
					//		隱藏『努力加載中』
					chatroom.selectors.$idRoomWhoType.find("span").text("").parent().hide();
					
				} else if ( "oldMessage" == obj.action ) {
					if ( obj.senderId != chatroom.opt.currentUserId ) {
						//	不是自己的頁面資料，不處理
						return;
					}
					
					//	聊天室舊資料倒入
					if ( obj.oldlist == null ) {
						//	沒資料了
						chatroom.opt.nomore = true;
					} else {
						chatroom.reload( obj.oldlist , function ( $clone ) {
							
							var $list = chatroom.selectors.$idCommentList;
							var index = $list.height();
							var $first = $( ".same-poster:first" , $list );

							$first.before( $( " > " , $clone ) );
					    	
							setScrollHeight( function(){
								chatroom.selectors.$chatroomMagListScroll.scrollTo( $list.height() - index - 26 );
							} );

							$('.time-ago-text').timeago();
						} );
					}
					
					//		隱藏『努力加載中』
					chatroom.selectors.$idRoomWhoType.find("span").text("").parent().hide();
					
				} else if ( "appeal" == obj.action ) {
					//	申訴
					toastr.info( i18n.view.appealResponse );
					modalBox.onClosc();
				}
			}
			
			$('.time-ago-text').timeago();
		}
	},
	reload : function(  list , callback ){
		/**		組訊息HTML	*/
		
		var size = list.length;
		
		var $clone = chatroom.selectors.$idCommentList.clone();
		
		$clone.find(" > .same-poster ").remove();
		
		var $more = $("#load-more-msg").clone();
		$more.removeClass("hidden").addClass( $more.attr("id") ).removeAttr("id");
		$clone.append( $more );
		
		var posterId = "";
		
		for ( var i = size -1 ; i >= 0 ; i-- ){
			var $box =  genCommentHtml( list[i] );
			
			var $laster = $(" .same-poster:last" , $clone );
			var preTZ = $(" .comment:last , unpay:last" , $laster ).attr("data-timestamp");
			var nowTZ = $(" .comment , .unpay" , $box ).attr("data-timestamp");
			
			var preDate = new Date( parseInt( preTZ ) ).toLocaleDateString(),
				nowDate = new Date( parseInt( nowTZ ) ).toLocaleDateString();
			
			var onCross = false;
			
			if ( ( !isNaN( parseInt( preDate ) ) && preDate != nowDate ) ||
					list[i].firstOfComment ) {
				
				if ( list[i].firstOfComment ) {
					//		聊天室的第一個訊息已出現
					chatroom.opt.nomore = true;
					$( ".load-more-msg" , $clone ).remove();
				}
				
				//		跨日，顯示日期線
				var $dateLine = $("#date-line").clone();
				
				var today = new Date().toLocaleDateString();
				var yestoday = new Date( new Date( today ).setDate( new Date( today ).getDate() - 1 ) ).toLocaleDateString();

				if ( nowDate == today ) {
					//		今天的分隔線
					$dateLine.removeClass("hidden").addClass( $dateLine.attr("id") ).removeAttr("id").find("div").html( i18n.view.dateLineToday );
				} else if ( nowDate == yestoday ) {
					//		今天的分隔線
					$dateLine.removeClass("hidden").addClass( $dateLine.attr("id") ).removeAttr("id").find("div").html( i18n.view.dateLineYestoday );
				}else {
					//		以前的日期分隔線
					$dateLine.removeClass("hidden").addClass( $dateLine.attr("id") ).removeAttr("id").find("div").html( list[i].timeString.split(" ")[0] );
				}

				$clone.append( $dateLine );
				onCross = true;
				
			}


			if ( posterId != list[i].senderId ) {
				//		換人發言
				$clone.append( $box );
				
			} else if ( onCross ) {
				//	跨日
				$clone.append( $box );
			} else {
				//		同個人發言
				
				if ( nowTZ - preTZ > 180000 ) {
					//		與上個訊息相差超過3分鐘
					$clone.append( $box );
				} else {
					$(" .commont-grop" , $laster ).append( $( ".comment" , $box ) );
					$(" .newest-comment-date" , $laster ).html( $(" .time-ago-text" , $box ) );
				}
			}
			
			posterId = list[i].senderId;

		}
		
		if ( callback ) {
			callback( $clone );
		}
	},
	changeRoom : function(roomId, callback){
		/**		換聊天室	*/
		chatroom.opt.roomId = roomId;
		chatroom.onstoptyping();
		callback();
	},
	sendmessage : function( msg , point ){
		/**		發送資料	*/
		var obj = chatroom.createMessagePack("sendMessage");
		obj.message = msg;

		chatroom.send(obj);
	},
	disconnected : function(){
		toastr.clear();
		var warnText = i18n.view.webSocketReconnect;
		warnText = warnText.replace("reconnectAttempts" , websocket.reconnectAttempts );
		toastr.warning( warnText );
	},
	closeWebsocket : function(){
		if(websocket) {
			websocket.close()
		};
	},
	onotherroom : function( obj ){
		/**		其他房間訊息		*/
		if ( "insert" == obj.action ) {
			var roomId = obj.roomId;
			
			//	我在看師父在線的分頁
			var $roomTab = chatroom.selectors.$chatroomItemList.find("[data-id="+ roomId +"]");
			
			//	是師父在線的訊息
			if ( $roomTab.length == 0 ) {
				//	沒有這個聊天室，建立
				$roomTab =  genChatroomItemHtml( obj );
				chatroom.selectors.$chatroomItemList.append( $roomTab );
			}
			
			var count = $roomTab.find(".chatroom-more").attr("data-count");
			count++;
			$roomTab.find(".chatroom-more").attr( "data-count" , count ).text( count + " new " );
		
			chatroom.opt.newestOtherSender = obj.senderName;
			
			resetTitleWithNewMsgCount();
		}
	},
	
	refreshTitle : function( obj ){
		/**		設定 document.title	*/
		var newmsg = obj.unReadCount;
		if ( interval ) {
			clearInterval( interval );
		}
		
		if ( newmsg > 0 ) {
			document.title = "(" + newmsg + ")"+chatroom.opt.title;
		} else {
			document.title = chatroom.opt.title;
			chatroom.opt.newestOtherSender = "";
		}
		
		if ( chatroom.opt.newestOtherSender != "" ) {
			var str = chatroom.opt.newestOtherSender + i18n.view.sendNewMsg;
			interval = setInterval( "flashTitle('"+document.title+"','"+str+"')" , 1000 );
		}

		var msg = newmsg;
		
		if ( msg > 0 ) {
			$(".chatroom-list-tab-item[data-tabIndex=0] span").text(" ("+msg+") ");
		} else {
			$(".chatroom-list-tab-item[data-tabIndex=0] span").text("");
		}

	},
	countUnRead : function(){
		/**		曲最新未讀數量		*/
		websocket.send("countUnRead");
	}
}

$(function(){
	
    //	變更螢幕高度，重新設定scroll高度
	$(window).resize(function(){
		
		if ( chatroom.selectors.$idCommentBox.css("position") == "absolute" ) {
			chatroom.selectors.$idIsComment.attr("rows", "2");
		} else {
			chatroom.selectors.$idIsComment.attr("rows", "3");
		}
		
		setScrollHeight();
	});
	
	//	檔案上傳
	$('#fileupload').fileupload({
		dataType : 'json',
		dropZone : chatroom.selectors.$idInsertComment,
		beforeSend : function( xhr ,data ){

			xhr.setRequestHeader('header_session_id_key', getCookie('X_SX_SESSION_ID') );

			if ( onFileUpload ) {
				return false;
			} else {
				onFileUpload = true;
			}
		},
		done : function(e, data) {
			lock = false;
			if( data && data.result.status.code != 1 ){
				toastr.error( data.result.status.message );
				resetFileupload();
			}else{
				$('#progress-bar-text').html( i18n.view.successUpload );
				toastr.info( i18n.view.fileOnProcess );
				
				var point = chatroom.selectors.$inputPoint.val();
				if ( point == "" ) {
					point = 0;
				}
				
				var obj = chatroom.createMessagePack("sendFileMessage");
				obj.message = data.result.fileId;
				obj.point = point;
				
				chatroom.send(obj);

				chatroom.selectors.$inputPoint.val("0");
				setTimeout( "resetFileupload()" , 1000 );
			}
		},
		progressall : function(e, data) {
			lock = true;
			$('#progress').show();
			$('#progress-bar-text').show();
			var progress = parseInt(data.loaded / data.total * 100, 10);
			$('#progress-bar').css('width', progress + '%');
	
			if(progress != 100){
				$('#progress-bar-text').html( i18n.view.uploading+"..." + progress + "%");
			}else{
				$img = $('#upload-ajax-loader-img');
				$('#progress-bar-text').html( i18n.view.handling + '...').append($img.show());
			}
		},
		fail : function( e , data ){
			toastr.error( i18n.view.fileUploadFail );
			resetFileupload();
		}
	});
	
});

/** 	設定 document.title	*/
function resetTitleWithNewMsgCount(){
	
	chatroom.countUnRead();
}

/**		輸入區鎖定與開啟	*/
function toggleInputArea(lock){
	var $idInsertComment = chatroom.selectors.$idInsertComment;
	if(lock){
		$idInsertComment.find("textarea").val('').attr('disabled', 'disabled').attr('placeholder', i18n.view.disconnectPlaceholder+'...').css('background-color', '#FCEFB8');
		$idInsertComment.find("input").attr('disabled', 'disabled');
		$idInsertComment.find("button").attr('disabled', 'disabled');
		$idInsertComment.find(".fileinput-button").fadeOut();
		$idInsertComment.find("button").fadeOut();
		colseEdit();
	}else{
		$idInsertComment.find("textarea").removeAttr('disabled').attr('placeholder', i18n.view.commentPlaceholder )
			.css('background-color', '#FFFFFF').val( $idInsertComment.find('textarea').attr("data-msg") );
		$idInsertComment.find("input").removeAttr('disabled');
		$idInsertComment.find("button").removeAttr('disabled');
		$idInsertComment.find(".fileinput-button").fadeIn();
		$idInsertComment.find("button").fadeIn();
	}
	//$idInsertComment.find("input[name=point]").attr('disabled', 'disabled');
}

/**		關閉所有修改區塊	*/
function colseEdit(){
	$(".comment-edit-cancel").each(function(){
		$(this).click();
	});
}


/**		取消修改	*/
$(document).on("click" , ".comment-edit-cancel" , function(){
	
	var $comment = $(this).parents(".comment");
	$comment.find(".comment-text").show();
	$comment.find(".edit-comment > ").remove();

});

/**		清空輸入欄位	*/
function resetInsert(){
	var $idInsertComment = chatroom.selectors.$idInsertComment;
	$idInsertComment.find("textarea").val("");
	if ( $(".appIframe").length == 0 ) {
		$idInsertComment.find("textarea").focus();
	}
	$idInsertComment.find("input").val("");
}

/**		產生單一訊息的HTML	*/
function genCommentHtml( obj ){
	
	var point = obj.point;
	var isposter = obj.senderId == chatroom.opt.currentUserId;
	var open = obj.open;
	
	var $box = $("#same-poster").clone();
	
	$box.attr( "data-sender" , isposter );
	
	//	發表人圖片
	$( ".poster-img" , $box ).attr( "src" , obj.senderImg );
	
	//	發表人名稱
	$( ".poster-name > a " , $box ).text( obj.senderName );
	
	if ( obj.saihuId != null && !isposter ) {
		var url = isUri;
		url = url.replace("saihuId", obj.saihuId );
		$( ".poster-info a " , $box ).attr( "href" , url );
	} else {

		$( ".poster-img" , $box ).replaceAll( $( ".poster-img" , $box ).parents("a") );
		$( ".poster-name > a " , $box ).replaceWith( $( ".poster-name > a " , $box ).text() );
	}
	
	//	發表時間
	var today = new Date().toLocaleDateString(),
		commentDay = new Date( parseInt( obj.timestamp ) ).toLocaleDateString();
	
	if ( today != commentDay ) {
		//		不是今天的訊息，顯示絕對時間
		$( ".time-ago-text" , $box ).text( obj.timeString.split(" ")[1]  );
	}else{
		//		今天的訊息，顯時相對時間
		$( ".time-ago-text" , $box ).attr( "title" , obj.timeString );
	}

	//	記錄訊息編號
	$( ".comment" , $box ).attr( "data-id" , obj.messageId ).attr("data-timestamp" , obj.timestamp );
	$( ".unpay" , $box ).attr( "data-id" , obj.messageId ).attr("data-timestamp" , obj.timestamp );
	
	$( ".comment" , $box ).attr( "data-point" , point );
	$( ".unpay" , $box ).attr( "data-point" , point );
	
	if ( obj.isImg || obj.isLink ) {
		//	非文字
		if ( obj.isImg ) {
			//	圖片
			$( ".comment-text" , $box ).remove();
			$( ".comment-file" , $box ).remove();
			
			$( ".comment-img > img" , $box ).attr("src" , obj.message );
			
		} else {
			//	檔案
			$( ".comment-text" , $box ).remove();
			$( ".comment-img" , $box ).remove();
			
			$( ".comment-file > a" , $box ).attr("href" , obj.message ).text( obj.fileName );
			
		}
		//	檔案沒有修改訊息
		$( ".open-comment-edit a.open-edit" , $box ).remove();
		
	} else {
		//	文字
		$( ".comment-text" , $box ).html( obj.message );
		$( ".comment-file" , $box ).remove();
		$( ".comment-img" , $box ).remove();
		
		//	文字沒有刪除檔案與修改價格
		$( ".open-comment-edit a.edit-price" , $box ).remove();
		$( ".open-comment-edit a.delete-file" , $box ).remove();
	}
	
	if ( isposter ) {
		//	我發佈的訊息
		$( ".unpay" , $box ).remove();
	
		//	我發佈的免費訊息
		$box.removeClass("need-point");
		$( ".glyphicon-usd" , $box ).remove();
		$( ".pay-status-remark" , $box ).remove();
		
		$( ".open-comment-edit a.edit-price" , $box ).remove();
		$( ".open-comment-edit a.open-tags" , $box ).remove();
		
	} else {
		//	對方發佈的訊息
		
		//	免費
		$( ".open-comment-edit" , $box ).remove();
		$box.removeClass("need-point");
		$( ".glyphicon-usd" , $box ).remove();
		$( ".pay-status-remark" , $box ).remove();
		$( ".unpay" , $box ).remove();
	}
	
	//	移除hidden跟id，加入same-poster
	$box.removeClass("hidden").addClass( $box.attr("id") ).removeAttr("id");
	
	return $box;
}

/**		設定scroll高度	*/
function setScrollHeight( callback ){
	var $commentList = chatroom.selectors.$idCommentList , $parent = $commentList.parent();
	$(".chatroom-msg-list-scroll-height").css("height" , $commentList.height() - $parent.height() );
	$parent.css( "position" , "relative");
	
	if (callback) {
		callback();
	}
}


/**		回復檔案上傳進度條	*/
function resetFileupload(){
	onFileUpload = false;
	$('#progress').hide();
	$('#progress-bar').css("width","0%");
	$('#progress-bar-text').html('');
}

/**		檔案拖曳效果	*/
$(document).bind('drop dragover', function (e) {
    e.preventDefault();
});

/**		只可輸入數字	*/
function intReplace( obj ){

	var $this = $(obj),
		str = $this.val(),
		int = IntegerFormate( str , 9999 );

	if ( str.length == 0 ) {
		$this.val( "" );
		return false;
	}

	if ( str != int || str.length != int.length ) {
		$this.val( int );
		return false;
	}
}

/**		換聊天室	*/
$(document).on("click" , ".chatroom-item , .my-issue-answer-room" , function(){
	
	var $this = $(this)
	
	chatroom.selectors.$idCommentBox.show();
	
	if ( onFileUpload || $this.hasClass("active") ) {
		//	檔案上傳中 或 同一間
		return false;
	}
	
	$(".chatroom-item").removeClass("active");
	$(".my-issue-answer-room").removeClass("active");
	$this.addClass("active");
	
	$this.find("i.fa-circle").remove();
	
	if ( $this.attr("data-best") == "true" ){
		$(".fa-star").show();
	} else {
		$(".fa-star").hide();
	}
	
	var newid = $this.attr("data-id");
	var issueid = $this.attr("data-issue");
	
	if ( issueid == null ) {
		issueid = chatroom.opt.issueId;
	}

	
	var otherName = $this.attr("data-name");
	var title = $this.attr("data-title");
	isUri = $this.attr("data-url");
	
	chatroom.selectors.$idCommentBox.block({message: '<h4>'+ i18n.view.hardLoading +'...</h4>'});
	
	chatroom.changeRoom( newid , function(){
		
		chatroom.selectors.$msgHeader.find(".msg-header-other-name").html( otherName );

		chatroom.opt.newestOtherSender = "";
		chatroom.opt.nomore = false;
		
		var obj = chatroom.createMessagePack("initMessage");
		chatroom.send(obj);
		
		chatroom.selectors.$idRoomWhoType.find("span").text( "努力加载中" ).parent().show();
		
		resetInsert();
		chatroom.selectors.$idCommentBox.unblock();
	} );
	
});

/**		shift + enter 換行	*/
$(document).on("keydown","#is-comment",function( event ){
	if ( event.keyCode == 13 ) {
		if ( !event.shiftKey ) {
			event.preventDefault();
			$("#submit-btn").click();
		}
	}
});

/**		shift + enter 換行	*/
$(document).on("keydown",".comment-edit-textarea",function( event ){
	if ( event.keyCode == 13 ) {
		if ( !event.shiftKey ) {
			event.preventDefault();
			$(this).val( $(this).val().trim() );
			$(this).parents(".comment").find(".comment-edit-submit").click();
		}
	}
});

/**		訊息送出	*/
$(document).on("click","#submit-btn",function(e){

	e.preventDefault();
	
	var $idIsComment = chatroom.selectors.$idIsComment;
	
	var msg = $idIsComment.val();
	var str = msg.trim();
	$idIsComment.val("");
	
	if ( str.length == 0 ) {
		if ( $(".appIframe").length == 0 ) {
			$idIsComment.focus();
		}
		toastr.clear();
		toastr.info( i18n.view.insertComment );
		return;
	}
	
	var point = chatroom.selectors.$inputPoint.val();

	if ( point == "" ) {
		point = 0;
	}

	chatroom.sendmessage( msg , point );
});

/**		是否停留在此分頁	*/
$(window).focus(function() {
	windowFocus = true;
	resetTitleWithNewMsgCount();
	var obj = chatroom.createMessagePack("readed");
	chatroom.send(obj);
}).blur(function() {
	windowFocus = false;
});


/**		顯示文字訊息修改視窗		*/
$(document).on( "click" , ".open-edit" , function(){

	//	關閉所有修改區塊。
	colseEdit();
	
	var $comment = $(this).parents(".comment");
	var $text = $comment.find(".comment-text");
	var commentId = $comment.attr("data-id");
	var point = $comment.attr("data-point");
	
	$text.hide();
	
	var value = $text.html();
	value = value.replace(/<br\s*\/?>/mg,"\n");

	var $box = $("#edit-comment").clone();
	
	if ( point && point > 0 ) {
		$('.comment-edit-point', $box ).val( point );
	} else {
		$('.comment-edit-point', $box ).remove();
	}

	$('textarea.comment-edit-textarea', $box ).val( value );
	$box.addClass( $box.attr("id") ).removeClass("hidden").removeAttr("id");
	
	$box.replaceAll( $( ".edit-comment" , $comment ) );
	
	
});


/**		確認修改	*/
$(document).on("click" , ".comment-edit-submit" , function(){
	
	var $box = $(this).parents(".edit-comment");
	var $comment = $box.parents(".comment");
	var msg = $box.find(".comment-edit-textarea").val();
	var point = $box.find(".comment-edit-point").val();
	
	if ( $comment.find(".comment-file").length != 0 ) {
		if ( !point ) {
			point = 0;
		}
	}

	var str = msg.trim();
	
	if ( str.length == 0 ) {
		toastr.clear();
		toastr.info( i18n.view.commentCantNull );
		return;
	}

	confirm( i18n.view.editCommentConfirm, function(result){
		if(result){
			var obj = chatroom.createMessagePack("editMessage");
			obj.messageId = $comment.attr("data-id");
			obj.message = msg;
			chatroom.send(obj);
			colseEdit();
		}
	});
});

/**		確認刪除	*/
$(document).on("click" , ".delete-file" , function(){
	
	//	關閉所有修改區塊
	colseEdit();
	var $comment = $(this).parents(".comment");
	confirm( i18n.view.deleteFileConfirm, function(result){
		if(result){
			var obj = chatroom.createMessagePack("deleteFileMessage");
			obj.messageId = $comment.attr("data-id");
			chatroom.send(obj);
		}
	});
});


/**		顯示 檔案 修改視窗	*/
$(document).on( "click" , ".edit-price" , function(){
	//	關閉所有修改區塊
	colseEdit();
	
	var $comment = $(this).parents(".comment");
	var commentId = $comment.attr("data-id");
	var point = $comment.attr("data-point");

	var value = $comment.find(".comment-file a").attr("href");
	if ( value == null ) {
		value = $comment.find(".comment-img img").attr("src");
	}
	
	var $box = $("#edit-comment").clone();

	$('.comment-edit-point', $box ).val( point );
	$('textarea.comment-edit-textarea', $box ).val( value ).hide();
	$box.addClass( $box.attr("id") ).removeClass("hidden").removeAttr("id");
	$box.replaceAll( $( ".edit-comment" , $comment ) );
	$box.find(".comment-edit-point").focus();
});

/**		開啟標記技能視窗		*/
$(document).on( "click" , ".open-tags" , function(){
	//	關閉所有修改區塊。
	colseEdit();
	var $comment = $(this).parents(".comment");
	openTagsBox( $comment.attr("data-id") );
	
});

/**		支付點數	*/
$(document).on("click" , ".pay-point" , function(){
	
	colseEdit();
	var $comment = $(this).parents("[data-id]");
	confirm( i18n.view.payPointConfirm, function(result){
		if(result){
			var obj = chatroom.createMessagePack("payPoint");
			obj.messageId = $comment.attr("data-id");
			chatroom.send(obj);
		}
	});
});

/**		換分頁	*/
$(document).on("click",".chatroom-list-tab-item",function(){
	
	var $this = $(this);
	
	if ( onFileUpload || $this.hasClass("active") ) {
		return;
	}

	chatroom.changeTab( $this , function(){
		
		$(".chatroom-list-tab-item").removeClass("active");
		$this.addClass("active");
		
		$("section > div:first")
			.removeClass("viewType0").removeClass("viewType1")
			.removeClass("viewType2").addClass("viewType"+chatroom.opt.tab);
	} );
});

/**		產生聊天室清單Html	*/
function genChatroomItemHtml( obj ){
	
	var $roomTab =  $("#chatroom-item-temp > ").clone();
	
	if ( chatroom.opt.tab == 2 ) {
		
		var str = obj.answerCount+" people";
		
		$roomTab.removeClass("chatroom-item");

		if ( obj.unReadCount == 0 ) {
			$( ".chatroom-issue-answer" , $roomTab ).attr( "data-count" , obj.answerCount ).text( str );
		} else {
			$( ".chatroom-issue-answer" , $roomTab ).attr( "data-count" , obj.answerCount ).html('<i class="fa fa-circle"></i>').find("i").text( str );
		}

		$roomTab.attr("data-id", obj.issueId);
	} else {
		$roomTab.removeClass("chatroom-issue-item");
		$( ".chatroom-more" , $roomTab ).attr( "data-count" , obj.unReadCount );
		if ( obj.unReadCount > 0 ) {
			$( ".chatroom-more" , $roomTab ).text( obj.unReadCount+" new" );
		}
		$roomTab.attr("data-id", obj.roomId)
	}
	
	var url = $roomTab.attr("data-url");
	
	if ( obj.saihuId != null && obj.saihuId != "" ) {
		url.replace( "saihuId" , obj.saihuId );
	} else {
		url = "";
	}
	
	$roomTab.attr("data-name" , obj.senderName )
		.attr("data-url" , url)
		.attr("data-title" , obj.issueTitle )
		.attr("data-issue" , obj.issueId );
	
	$(" img " , $roomTab ).attr("src", obj.senderImg );
	$(" .chatroom-other-name " , $roomTab ).text( obj.senderName );
	$(" .chatroom-issue-title " , $roomTab ).text( obj.issueTitle ).attr("title",obj.issueTitle);
	$(" .chatroom-issue-contexts " , $roomTab ).text( obj.issueContents ).attr("title",obj.issueContents);

	var date = new Date( obj.timestamp );
	var str = date.getFullYear()+"/"+(date.getMonth()+1)+"/"+date.getDate()+" "+date.getHours()+":"+date.getMinutes();
	$( ".chatroom-time" , $roomTab ).text( str );
	
	return $roomTab;
}

/**		換問題	*/
$(document).on("click",".chatroom-issue-item",function(){
	
	var $this = $(this)
	
	chatroom.selectors.$idCommentBox.show();
	
	if ( onFileUpload || $this.hasClass("active") ) {
		//	檔案上傳中 或 同一間
		return;
	}
	
	$(".chatroom-issue-item").removeClass("active");
	$this.addClass("active");
	
	var newid = $this.attr("data-id");

	chatroom.selectors.$idCommentBox.block({message: '<h4>'+ i18n.view.hardLoading +'...</h4>'});
	chatroom.changeIssue( newid , function(){

		chatroom.opt.newestOtherSender = "";
		chatroom.opt.nomore = false;
		
		var obj = chatroom.createMessagePack("changeIssue");
		chatroom.send(obj);
		
		resetInsert();
		chatroom.selectors.$idCommentBox.unblock();
	} );
	
});

/**		此分頁沒有聊天室，鎖住頁面	*/
function noRoom( msg ){
	
	chatroom.selectors.$msgHeader.find(".msg-header-other-name").html( "" );
	chatroom.selectors.$msgHeader.find(".msg-header-issue-title").html( "" ).attr("title","");
	
	toggleInputArea(true);
	chatroom.selectors.$idIsComment.val( msg );
}

/**		解開頁面	*/
function noRoomunBlock(){
	toggleInputArea(false);
}

/**		選最佳解	*/
$(document).on("click",".issue-best-answer .fa-star-o",function(){
	var $answer = chatroom.selectors.$issueAnswerList.find(".my-issue-answer-room.active");
	
	confirm( "是否選擇[ "+ $answer.attr("data-name") +" ]為最佳解", function(result){
		if(result){
			var obj = chatroom.createMessagePack("bestAnswer");
			chatroom.send(obj);
		}
	});
});


function showRoomList(){
	chatroom.selectors.$idCommentBox.hide();
}

function commentImgLoad( obj ){

	var $img = $(obj);
	var $isLaoading = $img.parent('.is-loading');
	$isLaoading.removeClass('is-loading');
	$('.fa', $isLaoading).remove();
	$img.fadeIn(800);
	
	
	var $this = $(obj);
	var loadingH = 144;
	var imgh = $this.height();
	var scrollNow = chatroom.selectors.$chatroomMagListScroll.scrollTop();
	
	setScrollHeight( function(){
		chatroom.selectors.$chatroomMagListScroll.scrollTo( scrollNow + ( imgh - loadingH ) );
	} );

}

//	加載資料
$(document).on("click",".load-more-msg > div",function(){
	var obj = chatroom.createMessagePack("oldMessage");
	var id = chatroom.selectors.$idCommentList.find("[data-id]:first").attr("data-id");
	obj.message = id;
	chatroom.send( obj );
	
	chatroom.selectors.$idRoomWhoType.find("span").text( "努力加载中" ).parent().show();
	
	$(this).parents(".load-more-msg").remove();
});